@@ -1,10 +1,30 @@
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
+/* wvWare
+ * Copyright (C) Caolan McNamara, Dom Lachowicz, and others
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
 #include <string.h>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <ctype.h>
 #include "wv.h"
 
 /*
@@ -1074,7 +1094,10 @@ wvApplysprmPIstdPermute (PAP * apap, U8 * pointer, U16 * pos)
 	    }
       }
     else
-	rgistd = NULL;
+      {
+      rgistd = NULL;
+      return;
+      }
     /*
        First check if pap.istd is greater than the istdFirst recorded in the sprm
        and less than or equal to the istdLast recorded in the sprm If not, the sprm
@@ -1659,7 +1682,8 @@ wvApplysprmCIstdPermute (CHP * achp, U8 * pointer, U16 * pos)
     (*pos) += 2;
     istdLast = dread_16ubit (NULL, &pointer);
     (*pos) += 2;
-    if ((cch - 6) / 2 != 0)
+
+    if (cch > 6)
       {
 	  rgistd = (U16 *) wvMalloc (sizeof (U16) * ((cch - 6) / 2));
 	  for (i = 0; i < (cch - 6) / 2; i++)
