@@ -1,11 +1,31 @@
+/* wvWare
+ * Copyright (C) Caolan McNamara, Dom Lachowicz, and others
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "wv.h"
 #include "getopt.h"
 
@@ -38,7 +58,7 @@ char *config = "wvHtml.xml";
 
 /* flags for -X / --xml option */
 int   xml_output = 0;
-char *xml_slash  = "";
+extern char *xml_slash;
 
 /* flag for disabling graphics */
 int   no_graphics = 0;
@@ -109,11 +129,6 @@ wvHtmlGraphic (wvParseStruct * ps, Blip * blip)
 		wvAppendStr (&name, ".bmp");
 		if (0 != HandleBitmap (ps, name, &blip->blip.bitmap))
 		    return (NULL);
-		remove_suffix (name, ".bmp");
-		if (ps->dir) chdir (ps->dir);
-		bmptopng (name);
-		if (ps->dir) chdir (wv_cwd);
-		wvAppendStr (&name, ".png");
 		return (name);
 	    }
       default:
@@ -169,7 +184,7 @@ HandleBitmap (wvParseStruct * ps, char *name, BitmapBlip * bitmap)
     if (ps->dir) chdir (wv_cwd);
     if (fd == NULL)
       {
-	fprintf (stderr,"\nCannot open %s for writing:%s\n",name,strerror (errno));
+	fprintf (stderr,"\nCannot open %s for writing\n",name);
 	exit (1);
       }
     size = wvStream_size (pwv);
@@ -189,20 +204,48 @@ HandleMetafile (wvParseStruct * ps, char *name, MetaFileBlip * bitmap)
     wvStream * pwv = bitmap->m_pvBits;
     FILE *fd = NULL;
     size_t size = 0, i;
+    U8 decompressf = 0;
 
     if (ps->dir) chdir (ps->dir);
     fd = fopen (name, "wb");
     if (ps->dir) chdir (wv_cwd);
     if (fd == NULL)
       {
-	fprintf (stderr,"\nCannot open %s for writing:%s\n",name,strerror (errno));
+	fprintf (stderr,"\nCannot open %s for writing\n",name);
 	exit (1);
       }
     size = wvStream_size (pwv);
     wvStream_rewind(pwv);
 
-    for (i = 0; i < size; i++)
-      fputc (read_8ubit(pwv), fd);
+    if (bitmap->m_fCompression == msocompressionDeflate)
+	decompressf = setdecom ();
+
+    if ( !decompressf)
+      {
+	for (i = 0; i < size; i++)
+	  fputc (read_8ubit(pwv), fd);
+      }
+    else /* decompress here */
+      {
+	  FILE *tmp = tmpfile ();
+	  FILE *out = tmpfile ();
+
+	  for (i = 0; i < size; i++)
+	    fputc (read_8ubit(pwv), tmp);
+
+	  rewind (tmp);
+	  decompress (tmp, out, bitmap->m_cbSave, bitmap->m_cb);
+	  fclose (tmp);
+
+	  rewind(out);
+
+	  for (i = 0; i < bitmap->m_cb; i++)
+	    fputc ( fgetc(out), fd);
+
+	  fclose(out);
+
+      }
+
     fclose (fd);
     wvTrace (("Name is %s\n", name));
     return (0);
@@ -358,8 +401,7 @@ main (int argc, char **argv)
     input = fopen (argv[optind], "rb");
     if (!input)
       {
-	  fprintf (stderr, "Failed to open %s: %s\n", argv[optind],
-		   strerror (errno));
+	fprintf (stderr, "Failed to open %s\n", argv[optind]);
 	  return (-1);
       }
     fclose (input);
@@ -453,6 +495,8 @@ main (int argc, char **argv)
     else if (ret != 0)
 	ret = -1;
     wvOLEFree (&ps);
+    wvShutdown ();
+
     return (ret);
 }
 
@@ -701,7 +745,7 @@ wvConvert_WMF_to_EPS (int width, int height, char **source)
     remove_suffix (sink, ".wmf");
     wvAppendStr (&sink, ".eps");
 
-    out = fopen (sink, "w");
+    out = fopen (sink, "wb");
 
     if (out == 0)
       {
@@ -918,7 +962,7 @@ wvConvert_PNG_to_EPS (int width, int height, char **source)
     remove_suffix (sink, ".png");
     wvAppendStr (&sink, ".eps");
 
-    out = fopen (sink, "w");
+    out = fopen (sink, "wb");
 
     if (out == 0)
       {
@@ -999,7 +1043,7 @@ wvConvert_JPG_to_EPS (int width, int height, char **source)
     remove_suffix (sink, ".jpg");
     wvAppendStr (&sink, ".eps");
 
-    out = fopen (sink, "w");
+    out = fopen (sink, "wb");
 
     if (out == 0)
       {
@@ -1263,7 +1307,6 @@ mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp)
 	  wvTrace (("field middle\n"));
 	  if (achp->fOle2)
 	    {
-		/*pps_entry *test; */
 		wvError (
 			 ("this field has an associated embedded object of id %x\n",
 			  achp->fcPic_fcObj_lTagObj));
